<?php
/**
 * Mail Mint
 *
 * @package /app/Admin/API/Controllers
 *
 * @since 1.5.0
 *
 * @author [MailMint Team]
 * @email [support@getwpfunnels.com]
 */

namespace MintMailPro\Mint\Admin\API\Controllers;

use MailMintPro\Mint\Internal\AbandonedCart\Helper\Common;
use MailMintPro\Mint\Internal\AbandonedCart\Helper\Model;
use MRM\Common\MrmCommon;
use WP_REST_Request;
use Mint\MRM\API\Controllers\BaseController;
use WC_Session_Handler;

/**
 * Manages  AbandonedCart API callbacks
 *
 * @package /app/API/Controllers
 * @since 1.5.0
 */
class AbandonedCartFrontendController extends BaseController {

	/**
	 * Summary: Updates the checkout data for the cart.
	 *
	 * Description: Updates the checkout data for the cart based on the API parameters received in the request.
	 *
	 * @access public
	 *
	 * @param WP_REST_Request $request The REST request object.
	 *
	 * @return WP_REST_Response Returns a REST response with the success status and message.
	 * @since 1.5.0
	 */
	public function update_checkout( WP_REST_Request $request ) {
		// Get the API parameters from the request.
		$params               = MrmCommon::get_api_params_values( $request );
		$woocommerce          = WC();
		$woocommerce->session = new WC_Session_Handler();
		$woocommerce->session->init();

		$cart_totals  = $woocommerce->session->cart_totals;
		$meta_details = Common::get_current_cart_details( $woocommerce->session );

		$cart_email   = isset( $params['email'] ) ? $params['email'] : '';
		$cart_details = Model::get_cart_details_by_key_and_status( 'email', $cart_email, array( 'pending', 'abandoned' ) );

		if ( ! empty( $cart_details ) ) {
			$session_key = $woocommerce->session->get_customer_id();
			$cart_id     = isset( $cart_details['id'] ) ? $cart_details['id'] : 0;
		} else {
			$session_key = $woocommerce->session->get_customer_id();
			// Get the cart details by session_key.
			$cart_details = Model::get_cart_details_by_key_and_status( 'session_key', $session_key, array( 'pending', 'abandoned' ) );
			$cart_id      = isset( $cart_details['id'] ) ? $cart_details['id'] : 0;
		}

		if ( empty( $cart_details ) ) {
			// Return error response if cart details are not found.
			return rest_ensure_response(
				array(
					'success' => false,
					'message' => __( 'Cart Details Not Found.', 'mailmint-pro' ),
				)
			);
		}

		$cart_details['session_key']   = $session_key;
		$cart_details['total']         = isset( $cart_totals['total'] ) ? $cart_totals['total'] : 0;
		$cart_details['updated_at']    = current_time( 'mysql' );
		$cart_details['email']         = isset( $cart_details['email'] ) ? sanitize_email( $params['email'] ) : '';
		$cart_details['checkout_data'] = isset( $cart_details['checkout_data'] ) ? maybe_serialize( array( 'fields' => json_decode( $params['checkout_fields_data'], true ) ) ) : '';
		// Update the cart details in the database.
		Model::update( $cart_details, $cart_id );
		Model::update_cart_meta( $cart_id, 'abandoned_cart_meta', maybe_serialize( $meta_details ) );

		// Return success response indicating that checkout data has been updated.
		return rest_ensure_response(
			array(
				'success' => true,
				'message' => __( 'Checkout data has been updated.', 'mailmint-pro' ),
			)
		);
	}

	/**
	 * Request for  AbandonedCart update chkout page
	 *
	 * @param WP_REST_Request $request equest object used to generate the response.
	 * @return array|\WP_Error|\WP_HTTP_Response|\WP_REST_Response
	 * @since 1.5.0
	 */
	public function delete_abandoned_cart( WP_REST_Request $request ) {
		$params = MrmCommon::get_api_params_values( $request );
		if ( empty( $params['email'] ) ) {
			return $this->get_error_response( __( 'Email not found.', 'mailmint-pro' ), 400 );
		}

		$cart_details = Model::get_cart_details_by_key( 'email', $params['email'], 'pending' );
		if ( !empty( $cart_details ) ) {
			$id        = !empty( $cart_details['id'] ) ? $cart_details['id'] : '';
			$is_delete = Model::delete( $id );
			if ( $is_delete ) {
				return $this->get_success_response( __( 'Deleted.', 'mailmint-pro' ), 200 );
			}
			return $this->get_error_response( __( 'Not Deleted.', 'mailmint-pro' ), 400 );
		}
		return $this->get_error_response( __( 'No cart details found.', 'mailmint-pro' ), 400 );
	}

	/**
	 * Default permission_callback
	 *
	 * @return bool
	 */
	public function rest_permissions_check() {
		return true;
	}
}
