<?php
/**
 * Mail Mint
 *
 * @author [MRM Team]
 * @email [support@rextheme.com]
 * @create date 2022-08-09 11:03:17
 * @modify date 2022-08-09 11:03:17
 * @package /app/Admin/API/Controllers
 */

namespace MintMailPro\Mint\Admin\API\Controllers;

use MailMint\App\Helper;
use MailMintPro\Mint\Internal\Traits\Singleton;
use MailPoet\AdminPages\Pages\Help;
use Mint\MRM\Admin\API\Controllers\MessageController;
use Mint\MRM\DataBase\Models\CampaignModel;
use Mint\MRM\DataBase\Models\ContactModel;
use Mint\MRM\DataBase\Models\EmailModel;
use MintMail\App\Internal\Automation\AutomationLogModel;
use MRM\Common\MrmCommon;
use WP_REST_Controller;
use WP_REST_Request;

/**
 * Manages Analytics API callbacks
 *
 * @package /app/API/Controllers
 * @since 1.0.0
 */
class AnalyticsController extends WP_REST_Controller {

	use Singleton;


	/**
	 * Request for campaign analytics on regular or sequence campaign
	 *
	 * @param WP_REST_Request $request Request object used to generate the response.
	 * @return WP_REST_Response
	 * @since 1.0.0
	 */
	public function campaign_analytics_request( WP_REST_Request $request ) {
		$reports     = array();
		$params      = MrmCommon::get_api_params_values( $request );
		$campaign_id = isset( $params['campaign_id'] ) ? $params['campaign_id'] : '';
		$campaign    = CampaignModel::get( $campaign_id );
		$status      = isset( $campaign['status'] ) ? $campaign['status'] : '';

		$recipients_meta = CampaignModel::get_campaign_meta_value( $campaign_id, 'recipients' );
		$recipients_meta = MrmCommon::filter_recipients( maybe_unserialize( $recipients_meta ), $status );
		$recipients      = isset( $recipients_meta['total_recipients'] ) ? $recipients_meta['total_recipients'] : 0;

		$title  = CampaignModel::get_campaign_value( $campaign_id, 'title' );
		$emails = self::get_campaign_analytics( $campaign_id, $recipients );

		$reports['success']    = true;
		$reports['title']      = $title;
		$reports['recipients'] = $recipients;
		$reports['emails']     = $emails;
		return rest_ensure_response( $reports );
	}


	/**
	 * Prepare analytics for regular or sequence campaign
	 *
	 * @param int $campaign_id Campaign ID.
	 * @param int $total_recipients Total number of recipients.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function get_campaign_analytics( $campaign_id, $total_recipients ) {
		// Get campaign emails information.
		$emails = CampaignModel::get_campaign_email_ids( $campaign_id );
		$type   = CampaignModel::get_campaign_type( $campaign_id );

		// Adjust total recipients for recurring campaigns.
		if ( 'recurring' === $type ) {
			$total_recipients = EmailModel::recurring_campaign_total_recipients( $campaign_id );
		}

		// Handle analytics calculation for automation campaigns separately.
		if ( 'automation' === $type ) {
			return AutomationLogModel::prepare_analytics_for_automation_sequence( $campaign_id, $emails );
		}

		// Get business settings default configuaration.
		$default_business_settings = MrmCommon::business_settings_default_configuration();
		$business_settings         = get_option( '_mrm_business_basic_info_setting', $default_business_settings );
		$business_settings         = is_array( $business_settings ) && ! empty( $business_settings ) ? $business_settings : $default_business_settings;

		// Mapping camapign analytics with emails.
		$emails = array_map(
			function( $email ) use ( $total_recipients, $business_settings ) {
				$email_id           = isset( $email['id'] ) ? $email['id'] : '';
				$email_body         = isset( $email[ 'body_data' ] ) ? html_entity_decode( $email[ 'body_data' ] ) : '';
				$email['body_data'] = MrmCommon::get_contact_placeholder_data( $email_body );

				// Calculate delivered and bounced emails.
				$total_delivered = MessageController::prepare_delivered_reports( $email_id, $total_recipients );
				$total_bounced   = MessageController::prepare_bounced_reports( $email_id, $total_recipients );
				$bounced         = isset( $total_bounced['total_bounced'] ) ? $total_bounced['total_bounced'] : '';
				$delivered       = isset( $total_delivered['total_delivered'] ) ? $total_delivered['total_delivered'] : '';

				// Calculate click and open rate.
				$open_rate  = MessageController::prepare_open_rate_reports( $email_id, $bounced, $total_recipients );
				$click_rate = MessageController::prepare_click_rate_reports( $email_id, $bounced, $total_recipients );

				// Calculate last 24 hours performance.
				$last_day = MessageController::prepare_last_day_reports( $email_id );

				// Calculate total open and click on user agent or device.
				$devices = MessageController::prepare_device_reports( $email_id );

				// Calculate total unsubscribe.
				$unsubscribe = MessageController::prepare_unsubscribe_reports( $email_id, $bounced, $delivered );

				// Calculate order reports.
				$orders = MessageController::prepare_order_reports( $email_id );

				return array_merge( $email, $total_delivered, $total_bounced, $open_rate, $click_rate, $last_day, $devices, $unsubscribe, $orders );
			},
			$emails
		);

		return $emails;
	}

	/**
	 * User accessability check for REST API
	 *
	 * @return bool
	 * @since 1.0.0
	 */
	public function rest_permissions_check() {
		return true;
	}

}
