<?php
/**
 * Automation manager
 *
 * @package MintMail\App\Internal\Automation
 */

namespace MailMintPro\App\Internal\Automation;

use MailMint\App\Helper;
use Mint\MRM\DataBase\Models\CampaignEmailBuilderModel;
use MintMail\App\Internal\Automation\HelperFunctions;
use MRM\Common\MrmCommon;
use Mint\Mrm\Internal\Traits\Singleton;
use Mint\MRM\DataBase\Models\EmailModel;
use MintMail\App\Internal\Automation\Action\AutomationAction;

/**
 * Automation manager
 *
 * @package MintMail\App\Internal\Automation
 */
class AutomationManager {

	/**
	 * Initialization
	 */
	public function __construct() {
		add_action( 'mint_process_sequence', array( $this, 'process_sequence_data' ), 10 );
	}


	/**
	 * Process sequence data;
	 *
	 * @param array $data Get All data.
	 */
	public function process_sequence_data( $data ) {
		/**
		 * Applies filters before processing an automation sequence.
		 *
		 * This function applies filters to the provided data before processing it
		 * in an automation sequence. The filters are hooked to the 'mailmint_before_automation_sequence_process' action.
		 *
		 * @param mixed $data The data to be processed in the automation sequence.
		 * @return mixed The filtered data after applying the filters.
		 */
		$data = apply_filters( 'mailmint_before_automation_sequence_process', $data );
		if ( !empty( $data['email_id'] ) ) {
			$default_business_settings = MrmCommon::business_settings_default_configuration();
			$business_settings         = get_option( '_mrm_business_basic_info_setting', $default_business_settings );
			$business_settings         = is_array( $business_settings ) && ! empty( $business_settings ) ? $business_settings : $default_business_settings;

			$email_data  = HelperFunctions::get_email_body_by_email_id( $data['email_id'] );
			$email       = HelperFunctions::get_campaign_email_by_email_id( $data['email_id'] );
			$email_body  = isset( $email_data['email_body'] ) ? html_entity_decode( $email_data['email_body'] ) : ''; // phpcs:ignore
			$domain_link = get_site_url();
			$rand_hash   = MrmCommon::get_rand_email_hash( $data['user_email'], $data['automation_id'] );
			$track_url   = add_query_arg(
				array(
					'mint'  => 1,
					'route' => 'open',
					'hash'  => $rand_hash,
				),
				$domain_link . '/wp-json/mrm/v1/analytics/'
			);
			$image_url   = "<img src= $track_url />";
			$email_body  = $email_body . $image_url;
			$email_body  = Helper::replace_url( $email_body, $rand_hash );
			$email_body  = Helper::replace_placeholder( $business_settings, $email_body, $data['user_email'], $rand_hash );
			// Prepare header information.
			$headers      = array(
				'MIME-Version: 1.0',
				'Content-type: text/html;charset=UTF-8',
			);
			$sender_name  = isset( $email['sender_name'] ) ? $email['sender_name'] : '';
			$from         = 'From: ' . $sender_name;
			$sender_mail  = $from . ' <';
			$sender_mail .= isset( $email['sender_email'] ) ? $email['sender_email'] : '';
			$sender_mail .= '>';
            $headers[]      = $sender_mail; //phpcs:ignore
			$reply_name   = isset( $email['reply_name'] ) ? $email['reply_name'] : '';
			$reply_email  = isset( $email['reply_email'] ) ? $email['reply_email'] : '';
			if ( $reply_name && $reply_email ) {
				$headers[] = 'Reply-To: ' . $reply_name . ' <' . $reply_email . '>';
			} elseif ( $reply_email ) {
				$headers[] = $reply_email;
			}

			$preview   = isset( $email['email_preview_text'] ) ? $email['email_preview_text'] : '';
			$headers[] = 'X-PreHeader: ' . $preview;

			$email_subject  = isset( $email['email_subject'] ) ? $email['email_subject'] : '';
			$email_subject  = Helper::replace_placeholder( $business_settings, $email_subject, $data['user_email'], $rand_hash );
			$scheduler_data = array(
				'body'           => $email_body,
				'subject'        => $email_subject,
				'receiver_email' => !empty( $data['user_email'] ) ? $data['user_email'] : '',
				'header'         => $headers,
			);
			/**
			 * Summary: Applies the 'mint_replace_abandoned_carts_placeholder' filter to replace abandoned carts placeholders in the email body.
			 *
			 * Description: This code section applies the 'mint_replace_abandoned_carts_placeholder' filter hook to the email body.
			 * The filter allows for customization of the replacement of abandoned carts placeholders with actual values in the email body.
			 *
			 * @param string $scheduler_data['body'] The email body to be filtered.
			 * @param array  $data            An array containing data to be passed to the filter hook.
			 *
			 * @return string The filtered email body with replaced abandoned carts placeholders.
			 * @since 1.5.0
			 */
			$scheduler_data['body'] = apply_filters( 'mint_automation_email_body', $scheduler_data['body'], $data['data'] );
			$is_sent                = $this->send_message( $scheduler_data );
			$contact_id             = HelperFunctions::get_contact_id_by_broadcast_table( $data['user_email'] );
			$payload                = array(
				'automation_id' => $data['automation_id'],
				'step_id'       => $data['step_id'],
				'email_id'      => $data['email_id'],
				'email_type'    => 'automation',
				'email_address' => !empty( $data['user_email'] ) ? $data['user_email'] : '',
				'contact_id'    => $contact_id,
				'email_hash'    => $rand_hash,
				'created_at'    => current_time( 'mysql' ),
				'updated_at'    => current_time( 'mysql' ),
			);
			if ( $is_sent ) {
				$payload['status'] = __( 'sent', 'mailmint-pro' );
			} else {
				$payload['status'] = __( 'failed', 'mailmint-pro' );
			}
			EmailModel::insert( $payload );

			$log_payload = array(
				'automation_id' => $data['automation_id'],
				'step_id'       => $data['step_id'],
				'status'        => 'completed',
				'identifier'    => $data['identifier'],
				'email'         => !empty( $data['user_email'] ) ? $data['user_email'] : '',
			);
			HelperFunctions::update_log( $log_payload );
		}
	}


	/**
	 * Send a message to contact
	 *
	 * @param mixed $data Single message object.
	 * @return bool
	 * @since 1.0.0
	 */
	public function send_message( $data ) {
		$to      = isset( $data['receiver_email'] ) ? $data['receiver_email'] : '';
		$subject = isset( $data['subject'] ) ? $data['subject'] : 'Welcome to WPF';
		$body    = isset( $data['body'] ) ? html_entity_decode( $data['body'] ) : ''; //phpcs:ignore
		$body    = str_replace( '</html>', CampaignEmailBuilderModel::get_email_footer_watermark() . '</html>', $body );
		$headers = isset( $data['header'] ) ? $data['header'] : '';
		if ( $to ) {
			try {
				return wp_mail( $to, $subject, $body, $headers );
			} catch ( \Exception $e ) {
				return false;
			}
		}
		return false;
	}

}
