<?php
/**
 * LearnDash triggers
 *
 * @package MintMail\App\Internal\Automation\Connector\Trigger;
 */

namespace MintMail\App\Internal\Automation\Connector\Trigger;

use MintMail\App\Internal\Automation\HelperFunctions;
use Mint\Mrm\Internal\Traits\Singleton;
/**
 * LearnDash triggers
 *
 * @package MintMail\App\Internal\Automation\Connector
 */
class LearnDashTriggers {

	use Singleton;

	/**
	 * Connector name
	 *
	 * @var $connector_name
	 * @since 1.6.1
	 */
	public $connector_name = 'LearnDash';

	/**
	 * Initialization of LearnDash hooks
	 */
	public function init() {
		add_action( 'learndash_update_course_access', array( $this, 'mint_after_enrolls_in_course' ), 10, 4 );
		add_action( 'learndash_course_completed', array( $this, 'mint_after_completes_a_course' ) );
		add_action( 'learndash_lesson_completed', array( $this, 'mint_after_completes_a_lesson' ), 9, 1 );
	}

	/**
	 * Validate settings based on LearnDash course conditions.
	 *
	 * @param array $step_data The data related to the automation step.
	 * @param array $data      The data associated with the trigger.
	 *
	 * @return bool True if the settings are valid, false otherwise.
	 * @since 1.6.1
	 */
	public function validate_settings( $step_data, $data ) {
		$allowed_trigger_names = array( 'learndash_enrolled_course', 'learndash_complete_course' );
		if ( isset( $data['trigger_name'] ) && in_array( $data['trigger_name'], $allowed_trigger_names, true ) ) {
			if ( isset( $step_data['automation_id'], $step_data['automation_id'] ) ) {
				$step_data = HelperFunctions::get_step_data( $step_data['automation_id'], $step_data['step_id'] );
			}

			if ( isset( $step_data['settings']['learn_dash_settings']['courses'] ) && !empty( $step_data['settings']['learn_dash_settings']['courses'] ) ) {
				$selected_courses = $step_data['settings']['learn_dash_settings']['courses'];
				$course_id        = $data['data']['course_id'];
				return in_array( $course_id, array_column( $selected_courses, 'value' ) );
			}
			return true;
		}

		if ( isset( $data['trigger_name'] ) && ( 'learndash_complete_lesson' === $data['trigger_name'] ) ) {
			if ( isset( $step_data['automation_id'], $step_data['automation_id'] ) ) {
				$step_data = HelperFunctions::get_step_data( $step_data['automation_id'], $step_data['step_id'] );
			}

			if ( isset( $step_data['settings']['learn_dash_settings']['lessons'] ) && !empty( $step_data['settings']['learn_dash_settings']['lessons'] ) ) {
				$selected_lessons = $step_data['settings']['learn_dash_settings']['lessons'];
				$lesson_id        = $data['data']['lesson_id'];
				return in_array( $lesson_id, array_column( $selected_lessons, 'value' ) );
			}
			return true;
		}

		return true;
	}

	/**
	 * Trigger an action after a user enrolls in a course.
	 *
	 * @param int   $user_id           The ID of the user enrolling in the course.
	 * @param int   $course_id         The ID of the course being enrolled in.
	 * @param array $course_access_list List of course access details (not used in this function).
	 * @param bool  $remove            Whether to remove the user (not used in this function).
	 *
	 * @since 1.6.1
	 */
	public function mint_after_enrolls_in_course( $user_id, $course_id, $course_access_list, $remove ) {
		if ( false !== $remove ) {
			return;
		}

		$user_data = get_userdata( $user_id );
		if ( $user_data ) {
			$data = array(
				'connector_name' => $this->connector_name,
				'trigger_name'   => 'learndash_enrolled_course',
				'data'           => array(
					'user_email' => isset( $user_data->data->user_email ) ? $user_data->data->user_email : '',
					'first_name' => isset( $user_data->first_name ) ? $user_data->first_name : '',
					'last_name'  => isset( $user_data->last_name ) ? $user_data->last_name : '',
					'course_id'  => $course_id,
				),
			);

			/**
			 * Perform an action named 'MINT_TRIGGER_AUTOMATION' with the provided data.
			 *
			 * This action triggers automation processes with the given data for handling automation events.
			 *
			 * @param mixed $data The data associated with the automation trigger.
			 * @since 1.6.1
			 */
			do_action( MINT_TRIGGER_AUTOMATION, $data );
		}
	}

	/**
	 * Trigger an action after a user completes a course in LearnDash.
	 *
	 * @param array $course An array containing information about the completed course, including user and course data.
	 * @since 1.6.1
	 */
	public function mint_after_completes_a_course( $course ) {
		$user_data = get_userdata( $course['user']->ID );
		if ( $user_data ) {
			$data = array(
				'connector_name' => $this->connector_name,
				'trigger_name'   => 'learndash_complete_course',
				'data'           => array(
					'user_email' => isset( $user_data->data->user_email ) ? $user_data->data->user_email : '',
					'first_name' => isset( $user_data->first_name ) ? $user_data->first_name : '',
					'last_name'  => isset( $user_data->last_name ) ? $user_data->last_name : '',
					'course_id'  => $course['course']->ID,
				),
			);

			/**
			 * Perform an action named 'MINT_TRIGGER_AUTOMATION' with the provided data.
			 *
			 * This action triggers automation processes with the given data for handling automation events.
			 *
			 * @param mixed $data The data associated with the automation trigger.
			 * @since 1.6.1
			 */
			do_action( MINT_TRIGGER_AUTOMATION, $data );
		}
	}

	/**
	 * Trigger an action after a user completes a lesson in LearnDash.
	 *
	 * @param array $lesson An array containing information about the completed lesson, including user, course, and lesson data.
	 * @since 1.6.1
	 */
	public function mint_after_completes_a_lesson( $lesson ) {
		$user_data = get_userdata( $lesson['user']->ID );
		if ( $user_data ) {
			$data = array(
				'connector_name' => $this->connector_name,
				'trigger_name'   => 'learndash_complete_lesson',
				'data'           => array(
					'user_email' => isset( $user_data->data->user_email ) ? $user_data->data->user_email : '',
					'first_name' => isset( $user_data->first_name ) ? $user_data->first_name : '',
					'last_name'  => isset( $user_data->last_name ) ? $user_data->last_name : '',
					'course_id'  => $lesson['course']->ID,
					'lesson_id'  => $lesson['lesson']->ID,
				),
			);

			/**
			 * Perform an action named 'MINT_TRIGGER_AUTOMATION' with the provided data.
			 *
			 * This action triggers automation processes with the given data for handling automation events.
			 *
			 * @param mixed $data The data associated with the automation trigger.
			 * @since 1.6.1
			 */
			do_action( MINT_TRIGGER_AUTOMATION, $data );
		}
	}
}

